
# q-expansion of newform 50.16.a.c, downloaded from the LMFDB on 23 May 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K.gens()[0]**i for i in range(len(poly_data))]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 50
weight = 16
poly_data = [0, 1]

# The basis for the coefficient ring is just the power basis
# in the root of the defining polynomial above.
hecke_ring_character_values = None
aps_data = [[128], [918], [0], [953554], [17783232], [-140533322], [-2998870746], [3255852500], [-6774812202], [-7340322690], [-115428411388], [-150300986906], [1841603525142], [-1510018315682], [-6093750843366], [8267412829038], [-23516883061980], [-3135369104278], [36030983954794], [52169735384172], [-69977143684082], [-135317670906760], [-427456158822882], [-446581617299190], [-181247411845826], [1592915084388942], [-1372379307724802], [-2221966128058926], [-3678477527356150], [486922628614758], [-1641222949828286], [9759485760739392], [1176153279353214], [23450442447647540], [30239218486554450], [28390478024925452], [30875216852479054], [59170358480280238], [-41909712348154206], [68055919562706078], [110140119077901180], [1870532094607262], [185883297811704972], [128077036975440358], [-114449946350210346], [-229969010429961160], [184150513863627152], [-417711371408876042], [208870687347791874], [73115242052276150], [-467231113986962922], [-883271348609391720], [455014267233989102], [-959950088086486848], [958486837364310534], [2623155461946338718], [464128220395473690], [327934721608002092], [-2341503860306667506], [-1877731070258569098], [-2845130502680550602], [-535043168427646842], [5098097245132958794], [-7825018994362236588], [2861357206631766598], [7453214422397410494], [5885634332902889432], [-2236418850603310946], [-1580371633321804806], [-6745518083434092370], [-13497696446037178962], [-14604009269419538520], [-17865367746308835446], [-6150345989863683122], [12565631551149686060], [-12201772284303640122], [-13300589724479274510], [2742414658660339894], [-3401875676691544638], [48397141147953801230], [78153250950620250060], [14766790067058430802], [-89692945045777749348], [113181836148192749278], [-51938013660415030600], [39024863919289600398], [23306180358626715270], [-139662634274535100106], [72915810638491033302], [8270748356289176878], [-10079137823883459606], [-27466330892508040680], [198771721723131986794], [14561445376337003712], [-107372122605450924460], [63037096982376647358], [-312583922055607895490], [-118550476595470352598], [-327493371317252846762], [-1747479561758716378], [-508895513472254872286], [186799992605193051414], [-650191494366188287002], [-225473703691560216690], [-296425479499775399848], [-314894907859894136426], [-90266099511295507566], [779095921040063161998], [567360465178398380160], [-1166084425519476190618], [1256512995353504781034], [682643036820469161958], [-818291797309653015786], [1565831374950933302780], [-1351696885186763073988], [1682094274199973016302], [444180352294663666918], [1615721742383984101434], [608831027994638144238], [1576525577594598981900], [-1067774019552568851838], [-1366374828975768360722], [-2907038148375871585386], [-740733377501756084802], [535532690697025872152], [3323942017062813889482], [-2002772347282128196090], [-4229302238013751695720], [1047182521779252934594], [61485593267697996238], [4006848174133714965020], [4558456320008470783158], [-3386812098132909394108], [-6565172382476953078466], [-1481421771153237059718], [-1985257081027514257150], [5636978337000651559878], [-2512946146145474409086], [3920096706074647888014], [-3885045191339224736790], [-4707144744955208595568], [-4610856852236278214238], [-4540206789523299629642], [8334056457735913652154], [-6214409209187019424150], [599543342294410271280], [5354315115999952639318], [-9253658799215536237026], [-1831957403564194100620], [-3512033724071238820602], [-11998942616664046049666], [-7701341867150263195698], [-17409631003035351116282], [9863547839813193017154], [-2234954765669549310326], [29336613622916159868852], [-3752996486728766314480], [-9108834335472180497130], [-5959601477671582633946], [28433649192900483677862], [23798410433602504382874], [-28267040539452633069642], [-27653879023009753066886], [-26528693513909536013448], [-27773696628407410929906], [-35225865009029602481922], [-3751408230002681917948], [-27744957994822441709906], [39156164145715572898130], [35965571660573076121398], [-39609677281796623144980], [-62145384777778013119258], [-1846451628595935671148], [74796463824857079968398], [44749765354155937206800], [61058279575339619823930], [-6595364577926592486568], [67496782242811166022462], [-10279162402900454101082], [-23362248904378360657510], [-117666506576857713484886], [-9262040304078355908528], [-6253317994589315661602], [57648182990700291661614], [31991280016790334237558], [3064956794517020264550], [77432362356578994612934], [-41426105791203185040602], [-149958646870807051275250], [-20425909859359527179148], [-103739777315860177864682], [-51710647750211942951202], [91063977831164955819272], [-82501638721899372222198], [-98457263324712333538686], [-78721570971531773509482], [-225718201279801370377618], [-192751014856917080498402], [38248611774435088773534], [29809085158667141723598], [215941040770539955936350], [-38333940837847119947428], [263675742762314440181374], [138837513426533416151570], [218109776511326901750900], [52942917725926491217854], [-145992733284627042545440], [-19501205960406024896202], [377815458177192663656490], [-22097633807151712338328], [106125933090460156307974], [160942204706375506102722], [134808210779137527008158], [-175472777725114323775566], [-458549329467394207181520], [199040664156211609033862], [-62134794361134996067886], [-36676814378148005874318], [-248670365534069269407486], [-71939820036249292896762], [358291731771602404063442], [-150077235900206223648160], [188027418421670221827090], [-367803648728013481637762], [-276853661177509633537806], [-28427637850248171184510], [60624494987937063085518], [-26063226385357364133120], [-323973092542337285124806], [-178786358193533672321928], [736353760551108992050918], [-367054046551567606222780], [-329175655722427477293508], [-416631669356765308929018], [-1169418131014029993257402], [1042719579005861192500674], [-519894929418818396643610], [126728695866961903903038], [157411463484100520566860], [-432855264911998253352348], [-8687176460520785996082], [970161796327603928771072], [904190279463798203883238], [332470076376212971656590], [-1237987154492937373151682], [1547678890736930968110120], [196736667745763940727474], [-659990301060310781882328], [-1439664416300287507294540], [246346978477390564330158], [1963602387465971369916454], [-270388343757182262873198], [1257104311312873092890634], [-423006035055117475767010], [-1713907249939429098157602], [1409214155289135771967620], [-470714542350258836406418], [410798621985169241366314], [-1021395757591492416282786], [19655241228683970038134], [-2009471526571691699030042], [1516992432056054053686834], [-1837672738219625792628490], [-723717289769061131293082], [1834289221025826576371094], [-1308045184597383042858460], [-1969548187226229962419170], [-577035063009761730918918], [3122045894877823333135438], [-1380014807438830483113162], [-1835079009673837101886858], [1995557121230043354681994], [1748583937397753452070998], [759533177573787023060240], [-3407204420360517287234186], [-1083350308825178990539562], [2982699900239960028373674], [-2464204396260831789624850], [261349274631823038196982], [-1131917143183528172925648], [4013026288114209499378518], [-3764488559217607219986268], [3282780194375811994893594], [-2951157912688359827994418], [1341292256075471704594594], [4614101716608653323137252], [2848514573341451632066438], [2361159414755829223994214], [6361820411993183459295920], [-2552688971411454855367530], [-1172169491110318893043878], [4891098729518893773837714], [1048904336669065419530718], [-560113802873855560179888], [-6225920499095281290807362], [413610744760099095638010], [4087843720207013254124852], [-4729168905664539887206122], [-8309287091338643106484020], [6109739413750205016995194], [-3075860726643422971994402], [6077347596571566285403854], [-8624980679077563493478440], [-1947992734734473995879002], [-4467951415792923448730848], [-10744944832088132276409026], [-3333106230703791989406726], [-2917890878303693232594610], [4389100678291557394484520], [-530809334952036813996122], [-4931847898862974687175082], [11772903712287745479501030], [12210270552621182199540642], [-8680526699685150214361402], [959866748878480278752754], [-3819235685306389166065990], [561156465983823174765300], [5836549288229256207241332], [-15437632169722042458965402], [16847197181476853893684470], [-12721678411642961961482728], [1681604068941315281278894], [-12367855186475401119403638], [6879929372552070550575838], [4147315583725521033628398], [-15791061037005952225203778], [1901597706902814236987780], [11616236817940723858675198], [1174124354312904990565194], [-17910493724739012853965762], [3718014241661701381582202], [14460146956245355459673934], [-12687360069414941466471640], [-1300455439246657800466482], [-5020555486004874198518128], [-18361585411018496494716366], [4167351461741496294250010], [23222095263448337006574438], [6063964483643616336158822], [10380498418218803851676194], [-19465728250275117131642], [27919460025268308228889734], [-21977764379985983635283070], [20506014399440248526146892], [-2944489563105569081488122], [-4183535278626475821815100], [33135426694717679744803442], [3301065825961887093445834], [522950961866029662935652], [11169036064972937503562934], [5649181552607449216098992], [-2317056726468797617208186], [1312584627887243898710202], [20317840334330381377561438], [34718176690167871266273830], [-10204141180160403938882562], [-15305512973247376957179720], [4961608067192244340210152], [-41365029980784677492328786], [41143071074228765454128838], [-4334582183149519565829506], [-42035196906992483699167878], [19141054049408706039798954], [-25369059519086909434810860], [5251142491689465225695554], [42362072298474115832389318], [-24479938479750807708873066], [55981437919759402911242638], [9638256393026819439150182], [-15925003294482792337925808], [55122277778452112535302540], [21429351494220903894801198], [67586932575803951472032370], [31332200876795916788346092], [60681238772781413973251494], [9157194152420834156429700], [-8700551595403251858759948], [-67531615256297444423275922], [10409650695338431883599590], [17360702890256031469965934], [-60908114125434711526426938], [4227451189699447894026078], [89191131245243505317280634], [-16316641244528575460520786], [14203087572039111482584340], [3419033410227210834923958], [90771277835891986635738332], [-85345120731007122722435306], [-69601567028688328851159122], [-84147745633896254716982166], [-83755609772506186833492490], [-26516790777215597146925922], [-37829361540380949388362540], [46868595612734838302344954], [77789893534413088168928892], [-96574817623264395086882], [-13499755276873586771366920], [-46475490508922727861909270], [-62860415633521252536084448], [12696488387814332617639842], [-1915320007940668797368230], [46988488055605859921073318], [-11295994147186827813471926], [-81156942432223972605620706], [-86284494752450447514655470], [129023725671627619868561612], [122629305382237333290007774], [24972305977803953337014322], [-37621816958295764192073602], [-317080372931691909327060], [-39479971119840669635813042], [-87040711567260867885725706], [80380964850204404987336598], [-50884455632486408058069088], [-83864781857036537332201346], [36820267669667542601348022], [98905576313023167155855520], [91407759950149744981254994], [19070546904583726534293054], [145891515603959283914666838], [51641291437216901267442570], [-89309679447997521474478106], [34824602338204521751393434], [-76980672886295522177214540], [145311188908154925932826118], [-129460723348203334246500546], [-111232215631676473017013722], [-185029291403998283151912210], [-122543290628648074849158088], [71902151814984983148985560]]
